#include <cstdlib>
#include <iostream>
#include <cstring>
#include <cerrno>
#include <regex>
#include <fstream>

#include "socklib.hpp"
#include "bufferedreaderwriter.hpp"

using namespace socklib;
using namespace std;

void
http_get_header(const string &serveur,
	 const string &port,
	 const string &chemin
	 ) {
  int code_retour;
  string message_erreur;
  string mime_type;
  int cont_length;
  bool chunked;
    
  int s = CreeSocketClient(serveur, port);
  BufferedReaderWriter web(s);
  
  ostringstream mess;
  
  mess << "HEAD " << chemin << " HTTP/1.1" << endl
       << "host: " << serveur << endl
       << endl;
  
  web.write(mess.str());
  
  // lecture de l'entête
  bool cont = true;
  
  while (cont) {
    string ligne = web.read_line();
    std::smatch match;
    

    if (ligne[ligne.size()-1] == '\n') {
      ligne.pop_back();
    }
    if (ligne[ligne.size()-1] == '\r') {
      ligne.pop_back();
    }
    cerr << "recu : '" << ligne << "'" << endl;
    if (ligne == "") {
      cont = false;
      continue;
    }
    if (regex_search(ligne, match, regex("^HTTP/1\\.1 (\\d\\d\\d) (.*)$"))) {
      // premire ligne
      code_retour = atoi(match[1].str().c_str());
      message_erreur = match[2].str();
    }
    if (regex_search(ligne, match, regex("^Content-Length: (\\d+)$"))) {
      cont_length = atoi(match[1].str().c_str());
    }
    if (regex_search(ligne, match, regex("^Content-Type: (.+)$"))) {
      mime_type = match[1].str();
    }
    if (regex_search(ligne, match, regex("^Transfer-Encoding: chunked$"))) {
      chunked = true;
    }
  }
  cout << "code retour : " << code_retour << endl;
  cout << "mime type : " << mime_type << endl;
  if (chunked) {
    cout << "La page est séparée en morceaux" << endl;
  } else {
    cout << "La taille de la page est " << cont_length << endl;
  }
}

int main(int argc, char *argv[]) {

  if (argc != 2) {
    cerr << "usage : " << argv[0] << " <url>" << endl;
    exit(1);
  }

  string url(argv[1]);
  string server;
  string port;
  string chemin;
  smatch m;

  cout << "url = " << url << endl;
  if (regex_search(url, m, regex("^http://([^/]+)/?(.*)$"))) {
    
    smatch m1;
    string serv = m[1].str();
    if (regex_search(serv, m1, regex("^(.*):(.*)$"))) {
      server = m1[1].str();
      port   = m1[2].str();
    } else {
      server = serv;
      port   = "80";
    }
    chemin = "/"+m[2].str();

    cout << "server " << server << endl;
    cout << "port " << port << endl;
    cout << "chemin " << chemin << endl;
  } else {
    cerr << "Je n'arrive pas à lire l'url" << endl;
    exit(1);
  }
  
  http_get_header(server, port, chemin);

  return 0;
}
